@@ -59,7 +59,7 @@ pub(crate) fn convert_reassignment_to_asm(
                     .lhs
                     .iter()
                     .fold(reassignment.lhs[0].span(), |acc, this| {
-                        crate::utils::join_spans(acc, this.span())
+                        sway_types::span::join_spans(acc, this.span())
                     }),
                 format!(
                     "variable {} reassignment",
@@ -113,7 +113,7 @@ pub(crate) fn convert_reassignment_to_asm(
                     }
                 };
                 // TODO(static span) use spans instead of strings below
-                let span = crate::Span {
+                let span = sway_types::span::Span {
                     span: pest::Span::new(
                         "TODO(static span): use Idents instead of Strings".into(),
                         0,
@@ -189,7 +189,7 @@ pub(crate) fn convert_reassignment_to_asm(
                         ptr.clone(),
                         return_register,
                         offset_in_words,
-                        crate::utils::join_spans(
+                        sway_types::span::join_spans(
                             reassignment.lhs[0].span(),
                             reassignment.rhs.span.clone(),
                         ),
