@@ -94,6 +94,14 @@ impl<T> From<Result<T, TypeError>> for CompileResult<T> {
 }
 
 impl<T> CompileResult<T> {
+    pub fn new(value: Option<T>, warnings: Vec<CompileWarning>, errors: Vec<CompileError>) -> Self {
+        CompileResult {
+            value,
+            warnings,
+            errors,
+        }
+    }
+
     pub fn ok(
         mut self,
         warnings: &mut Vec<CompileWarning>,
@@ -382,6 +390,8 @@ pub enum CompileError {
         span: Span,
         err: pest::error::Error<Rule>,
     },
+    #[error("Error parsing input: {err:?}")]
+    ParseError { span: Span, err: String },
     #[error(
         "Invalid top-level item: {0:?}. A program should consist of a contract, script, or \
          predicate at the top level."
@@ -714,8 +724,8 @@ pub enum CompileError {
     MoreThanOneEnumInstantiator { span: Span, ty: String },
     #[error("This enum variant represents the unit type, so it should not be instantiated with any value.")]
     UnnecessaryEnumInstantiator { span: Span },
-    #[error("Trait \"{name}\" does not exist in this scope.")]
-    TraitNotFound { name: Ident, span: Span },
+    #[error("Cannot find trait \"{name}\" in this scope.")]
+    TraitNotFound { name: String, span: Span },
     #[error("This expression is not valid on the left hand side of a reassignment.")]
     InvalidExpressionOnLhs { span: Span },
     #[error(
@@ -817,6 +827,28 @@ pub enum CompileError {
     IntegerContainsInvalidDigit { span: Span, ty: String },
     #[error("Unexpected alias after an asterisk in an import statement.")]
     AsteriskWithAlias { span: Span },
+    #[error("A trait cannot be a subtrait of an ABI.")]
+    AbiAsSupertrait { span: Span },
+    #[error("The name \"{fn_name}\" is defined multiple times for trait \"{trait_name}\".")]
+    NameDefinedMultipleTimesForTrait {
+        fn_name: String,
+        trait_name: String,
+        span: Span,
+    },
+    #[error("The trait \"{supertrait_name}\" is not implemented for type \"{type_name}\"")]
+    SupertraitImplMissing {
+        supertrait_name: String,
+        type_name: String,
+        span: Span,
+    },
+    #[error(
+        "Implementation of trait \"{supertrait_name}\" is required by this bound in \"{trait_name}\""
+    )]
+    SupertraitImplRequired {
+        supertrait_name: String,
+        trait_name: String,
+        span: Span,
+    },
 }
 
 impl std::convert::From<TypeError> for CompileError {
@@ -913,6 +945,7 @@ impl CompileError {
             Unimplemented(_, span) => span,
             TypeError(err) => err.internal_span(),
             ParseFailure { span, .. } => span,
+            ParseError { span, .. } => span,
             InvalidTopLevelItem(_, span) => span,
             Internal(_, span) => span,
             InternalOwned(_, span) => span,
@@ -1018,6 +1051,10 @@ impl CompileError {
             IntegerTooSmall { span, .. } => span,
             IntegerContainsInvalidDigit { span, .. } => span,
             AsteriskWithAlias { span, .. } => span,
+            AbiAsSupertrait { span, .. } => span,
+            NameDefinedMultipleTimesForTrait { span, .. } => span,
+            SupertraitImplMissing { span, .. } => span,
+            SupertraitImplRequired { span, .. } => span,
         }
     }
 
