@@ -3,12 +3,12 @@ use crate::Ident;
 use crate::{type_engine::TypeId, TypeParameter};
 
 #[derive(Clone, Debug)]
-pub(crate) struct TypedStructExpressionField<'sc> {
-    pub(crate) name: Ident<'sc>,
-    pub(crate) value: TypedExpression<'sc>,
+pub(crate) struct TypedStructExpressionField {
+    pub(crate) name: Ident,
+    pub(crate) value: TypedExpression,
 }
 
-impl TypedStructExpressionField<'_> {
+impl TypedStructExpressionField {
     pub(crate) fn copy_types(&mut self, type_mapping: &[(TypeParameter, TypeId)]) {
         self.value.copy_types(type_mapping);
     }
