@@ -90,6 +90,15 @@ pub(crate) enum TypedExpressionVariant {
         // this span may be used for errors in the future, although it is not right now.
         span: Span,
     },
+    SizeOf {
+        variant: SizeOfVariant,
+    },
+}
+
+#[derive(Clone, Debug)]
+pub(crate) enum SizeOfVariant {
+    Type(TypeId),
+    Val(Box<TypedExpression>),
 }
 
 #[derive(Clone, Debug)]
@@ -203,6 +212,12 @@ impl TypedExpressionVariant {
                     tag
                 )
             }
+            TypedExpressionVariant::SizeOf { variant } => match variant {
+                SizeOfVariant::Val(exp) => format!("size_of_val({:?})", exp.pretty_print()),
+                SizeOfVariant::Type(type_name) => {
+                    format!("size_of({:?})", type_name.friendly_type_str())
+                }
+            },
         }
     }
     /// Makes a fresh copy of all type ids in this expression. Used when monomorphizing.
@@ -317,6 +332,10 @@ impl TypedExpressionVariant {
                 };
             }
             AbiCast { address, .. } => address.copy_types(type_mapping),
+            SizeOf { variant } => match variant {
+                SizeOfVariant::Type(_) => (),
+                SizeOfVariant::Val(exp) => exp.copy_types(type_mapping),
+            },
         }
     }
 }
