@@ -12,10 +12,7 @@ use crate::{
 
 use sway_types::span::{join_spans, Span};
 
-use std::{
-    collections::{HashMap, HashSet},
-    sync::Arc,
-};
+use std::sync::Arc;
 
 pub(crate) use crate::semantic_analysis::ast_node::declaration::ReassignmentLhs;
 
@@ -136,7 +133,6 @@ impl TypedAstNode {
             self_type,
             build_config,
             dead_code_graph,
-            dependency_graph,
             opts,
             ..
         } = arguments;
@@ -165,7 +161,6 @@ impl TypedAstNode {
                 self_type,
                 build_config,
                 dead_code_graph,
-                dependency_graph,
                 mode: Mode::NonAbi,
                 opts,
             })
@@ -196,13 +191,7 @@ impl TypedAstNode {
                     // Import the file, parse it, put it in the namespace under the module name (alias or
                     // last part of the import by default)
                     let _ = check!(
-                        import_new_file(
-                            a,
-                            namespace,
-                            build_config,
-                            dead_code_graph,
-                            dependency_graph
-                        ),
+                        import_new_file(a, namespace, build_config, dead_code_graph),
                         return err(warnings, errors),
                         warnings,
                         errors
@@ -226,7 +215,6 @@ impl TypedAstNode {
                                     });
                                     insert_type(TypeInfo::ErrorRecovery)
                                 });
-
                             let result = {
                                 TypedExpression::type_check(TypeCheckArguments {
                                     checkee: body,
@@ -238,7 +226,6 @@ impl TypedAstNode {
                                     self_type,
                                     build_config,
                                     dead_code_graph,
-                                    dependency_graph,
                                     mode: Mode::NonAbi,
                                     opts,
                                 })
@@ -318,7 +305,6 @@ impl TypedAstNode {
                                     build_config,
                                     dead_code_graph,
                                     mode: Mode::NonAbi,
-                                    dependency_graph,
                                     opts
                                 }),
                                 error_recovery_function_declaration(fn_decl),
@@ -336,6 +322,7 @@ impl TypedAstNode {
                             interface_surface,
                             methods,
                             type_parameters,
+                            supertraits,
                             visibility,
                         }) => {
                             // type check the interface surface
@@ -345,6 +332,27 @@ impl TypedAstNode {
                                 warnings,
                                 errors
                             );
+
+                            // Error checking. Make sure that each supertrait exists and that none
+                            // of the supertraits are actually an ABI declaration
+                            for supertrait in &supertraits {
+                                match namespace
+                                    .get_call_path(&supertrait.name)
+                                    .ok(&mut warnings, &mut errors)
+                                {
+                                    Some(TypedDeclaration::TraitDeclaration(_)) => (),
+                                    Some(TypedDeclaration::AbiDeclaration(_)) => {
+                                        errors.push(CompileError::AbiAsSupertrait {
+                                            span: name.span().clone(),
+                                        })
+                                    }
+                                    _ => errors.push(CompileError::TraitNotFound {
+                                        name: supertrait.name.span().as_str().to_string(),
+                                        span: name.span().clone(),
+                                    }),
+                                }
+                            }
+
                             let trait_namespace = create_new_scope(namespace);
                             // insert placeholder functions representing the interface surface
                             // to allow methods to use those functions
@@ -352,6 +360,7 @@ impl TypedAstNode {
                                 CallPath {
                                     prefixes: vec![],
                                     suffix: name.clone(),
+                                    is_absolute: false,
                                 },
                                 TypeInfo::SelfType,
                                 interface_surface
@@ -368,7 +377,6 @@ impl TypedAstNode {
                                     insert_type(TypeInfo::SelfType),
                                     build_config,
                                     dead_code_graph,
-                                    dependency_graph
                                 ),
                                 vec![],
                                 warnings,
@@ -380,6 +388,7 @@ impl TypedAstNode {
                                     interface_surface,
                                     methods,
                                     type_parameters,
+                                    supertraits,
                                     visibility,
                                 });
                             namespace.insert(name, trait_decl.clone());
@@ -395,7 +404,6 @@ impl TypedAstNode {
                                         self_type,
                                         build_config,
                                         dead_code_graph,
-                                        dependency_graph,
                                         // this is unused by `reassignment`
                                         return_type_annotation: insert_type(TypeInfo::Unknown),
                                         help_text: Default::default(),
@@ -416,7 +424,6 @@ impl TypedAstNode {
                                 crate_namespace,
                                 build_config,
                                 dead_code_graph,
-                                dependency_graph,
                                 opts,
                             ),
                             return err(warnings, errors),
@@ -473,7 +480,6 @@ impl TypedAstNode {
                                         build_config,
                                         dead_code_graph,
                                         mode: Mode::NonAbi,
-                                        dependency_graph,
                                         opts
                                     }),
                                     continue,
@@ -484,6 +490,7 @@ impl TypedAstNode {
                             let trait_name = CallPath {
                                 prefixes: vec![],
                                 suffix: Ident::new_with_override("r#Self", block_span.clone()),
+                                is_absolute: false,
                             };
                             namespace.insert_trait_implementation(
                                 trait_name.clone(),
@@ -577,7 +584,6 @@ impl TypedAstNode {
                                     self_type,
                                     build_config,
                                     dead_code_graph,
-                                    dependency_graph
                                 ),
                                 vec![],
                                 warnings,
@@ -613,7 +619,6 @@ impl TypedAstNode {
                             self_type,
                             build_config,
                             dead_code_graph,
-                            dependency_graph,
                             mode: Mode::NonAbi,
                             opts
                         }),
@@ -637,7 +642,6 @@ impl TypedAstNode {
                                 self_type,
                                 build_config,
                                 dead_code_graph,
-                                dependency_graph,
                                 mode: Mode::NonAbi,
                                 opts
                             }),
@@ -658,7 +662,6 @@ impl TypedAstNode {
                             self_type,
                             build_config,
                             dead_code_graph,
-                            dependency_graph,
                             mode: Mode::NonAbi,
                             opts,
                         }),
@@ -680,7 +683,6 @@ impl TypedAstNode {
                             self_type,
                             build_config,
                             dead_code_graph,
-                            dependency_graph,
                             mode: Mode::NonAbi,
                             opts
                         }),
@@ -701,7 +703,6 @@ impl TypedAstNode {
                             self_type,
                             build_config,
                             dead_code_graph,
-                            dependency_graph,
                             mode: Mode::NonAbi,
                             opts,
                         }),
@@ -751,7 +752,6 @@ fn import_new_file(
     namespace: NamespaceRef,
     build_config: &BuildConfig,
     dead_code_graph: &mut ControlFlowGraph,
-    dependency_graph: &mut HashMap<String, HashSet<String>>,
 ) -> CompileResult<()> {
     let mut warnings = vec![];
     let mut errors = vec![];
@@ -801,13 +801,7 @@ fn import_new_file(
         namespace: module,
         ..
     } = check!(
-        crate::compile_inner_dependency(
-            file_as_string,
-            dep_namespace,
-            dep_config,
-            dead_code_graph,
-            dependency_graph
-        ),
+        crate::compile_inner_dependency(file_as_string, dep_namespace, dep_config, dead_code_graph),
         return err(warnings, errors),
         warnings,
         errors
@@ -833,7 +827,6 @@ fn reassignment(
         self_type,
         build_config,
         dead_code_graph,
-        dependency_graph,
         opts,
         ..
     } = arguments;
@@ -888,7 +881,6 @@ fn reassignment(
                     self_type,
                     build_config,
                     dead_code_graph,
-                    dependency_graph,
                     mode: Mode::NonAbi,
                     opts
                 }),
@@ -927,7 +919,6 @@ fn reassignment(
                         self_type,
                         build_config,
                         dead_code_graph,
-                        dependency_graph,
                         mode: Mode::NonAbi,
                         opts
                     }),
@@ -991,7 +982,6 @@ fn reassignment(
                     self_type,
                     build_config,
                     dead_code_graph,
-                    dependency_graph,
                     mode: Mode::NonAbi,
                     opts,
                 }),
@@ -1083,7 +1073,6 @@ fn type_check_trait_methods(
     self_type: TypeId,
     build_config: &BuildConfig,
     dead_code_graph: &mut ControlFlowGraph,
-    dependency_graph: &mut HashMap<String, HashSet<String>>,
 ) -> CompileResult<Vec<TypedFunctionDeclaration>> {
     let mut warnings = vec![];
     let mut errors = vec![];
@@ -1221,7 +1210,6 @@ fn type_check_trait_methods(
                 self_type,
                 build_config,
                 dead_code_graph,
-                dependency_graph,
                 mode: Mode::NonAbi,
                 opts: TCOpts { purity }
             }),
