@@ -251,8 +251,13 @@ impl Dependencies {
                 interface_surface,
                 methods,
                 type_parameters,
+                supertraits,
                 ..
             }) => self
+                .gather_from_iter(supertraits.iter(), |deps, sup| {
+                    deps.gather_from_call_path(&sup.name, false, false)
+                        .gather_from_traits(&sup.type_parameters)
+                })
                 .gather_from_iter(interface_surface.iter(), |deps, sig| {
                     deps.gather_from_iter(sig.parameters.iter(), |deps, param| {
                         deps.gather_from_typeinfo(&param.r#type)
@@ -368,8 +373,9 @@ impl Dependencies {
                 fields,
                 ..
             } => {
-                self.deps
-                    .insert(DependentSymbol::Symbol(struct_name.as_str().to_string()));
+                self.deps.insert(DependentSymbol::Symbol(
+                    struct_name.suffix.as_str().to_string(),
+                ));
                 self.gather_from_iter(fields.iter(), |deps, field| {
                     deps.gather_from_expr(&field.value)
                 })
@@ -414,6 +420,8 @@ impl Dependencies {
             }
             Expression::TupleIndex { prefix, .. } => self.gather_from_expr(prefix),
             Expression::DelayedMatchTypeResolution { .. } => self,
+            Expression::SizeOfVal { exp, .. } => self.gather_from_expr(exp),
+            Expression::SizeOfType { .. } => self,
         }
     }
 
