@@ -408,6 +408,9 @@ impl TypeInfo {
         };
         ok(name, vec![], vec![])
     }
+    pub(crate) fn size_in_bytes(&self, err_span: &Span) -> Result<u64, CompileError> {
+        Ok(self.size_in_words(err_span)? * 8)
+    }
     /// Calculates the stack size of this type, to be used when allocating stack memory for it.
     pub(crate) fn size_in_words(&self, err_span: &Span) -> Result<u64, CompileError> {
         match self {
